#!/usr/bin/env python3

# from . import colors
from . import ax
from ._add_hue import add_hue
from ._annotated_heatmap import annotated_heatmap
from ._configure_mpl import configure_mpl
from ._draw_a_cube import draw_a_cube
from ._get_RGBA_from_colormap import get_RGBA_from_colormap
from ._mk_colorbar import mk_colorbar
from ._mk_patches import mk_patches
from ._subplots import subplots
